unit winutils;

{$mode objfpc}{$H+}

interface

uses
  LclType, Classes, SysUtils, Windows, Win9xWsManager;

(*
{ ShellExecute() and ShellExecuteEx() error codes }

{ regular WinExec() codes }
  SE_ERR_FNF              = 2;       { file not found }
  SE_ERR_PNF              = 3;       { path not found }
  SE_ERR_ACCESSDENIED     = 5;       { access denied }
  SE_ERR_OOM              = 8;       { out of memory }
  SE_ERR_DLLNOTFOUND      = 32;

{ error values for ShellExecute() beyond the regular WinExec() codes }
  SE_ERR_SHARE                    = 26;
  SE_ERR_ASSOCINCOMPLETE          = 27;
  SE_ERR_DDETIMEOUT               = 28;
  SE_ERR_DDEFAIL                  = 29;
  SE_ERR_DDEBUSY                  = 30;
  SE_ERR_NOASSOC                  = 31;
*)

function ExecuteFile(const FileName: string): Boolean;
function ExecuteFileEx(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): Boolean;
function ExecutePrintFile(const FileName: String): Boolean;

function DiskTypeStr(Drive: Char): ShortString;
function DiskInDrive(Drive: Char): Boolean;

{$I csidl-consts.inc}

//Just to make this function public,and add an extra parameter
Function GetSpecialDir(ID :  Integer; const ForceCreate: Boolean = False) : String;


implementation

function ExecuteFileEx(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  {$PUSH}{$HINTS OFF}
  AHandle := ShellExecute(LongInt(nil), nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
  if AHandle <= 32 then Result := False else Result := True;
  {$POP}
end;

const
  VER_PLATFORM_WIN32_NT = 2;

function ExecutePrintFile(const FileName: String): Boolean;
var
{$IFDEF WinCE}
  Info: SHELLEXECUTEINFO;
{$ELSE}
  ws: WideString;
  ans: AnsiString;
{$ENDIF}
begin
  Result := False;
  if FileName = '' then Exit;
  {$IFDEF WinCE}
  FillChar(Info, SizeOf(Info), 0);
  Info.cbSize := SizeOf(Info);
  Info.fMask := SEE_MASK_FLAG_NO_UI;
  Info.lpVerb := 'print';
  Info.lpFile := PWideChar(UTF8Decode(AURL));
  Result := ShellExecuteEx(@Info);
  {$ELSE}
  if Win32Platform = VER_PLATFORM_WIN32_NT then
  begin
    ws := UTF8Decode(FileName);
    Result := ShellExecuteW(0, 'print', PWideChar(ws), nil, nil, SW_SHOWNORMAL) > 32;
  end
  else
  begin
    ans := Utf8ToAnsi(FileName); // utf8 must be converted to Windows Ansi-codepage
    Result := ShellExecute(0, 'print', PAnsiChar(ans), nil, nil, SW_SHOWNORMAL) > 32;
  end;
  {$ENDIF}
end;


function ExecuteFile(const FileName: string): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  {$PUSH}{$HINTS OFF}
  AHandle := ShellExecute(LongInt(nil), nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, ''),
    StrPCopy(zDir, ''), SW_SHOWNORMAL);
  if AHandle <= 32 then Result := False else Result := True;
  {$POP}
end;



(*
procedure CaptureScreenRect( ARect: TRect; ABitmap: TBitmap );
var
  ScreenDC: HDC;
begin
  with ABitmap, ARect do
  begin
    Width := Right - Left;
    Height := Bottom - Top;
    ScreenDC := GetDC( 0 );
    try
      BitBlt( Canvas.Handle, 0, 0, Width, Height, ScreenDC,
        Left, Top, SRCCOPY );
    finally
      ReleaseDC( 0, ScreenDC );
    end;
  end;
end;
*)
(*
{ Use this to capture the entire screen... }
procedure CaptureScreen;
begin
  with Screen do
    CaptureScreenRect( Rect( 0, 0, Width, Height ), Form1.Image1.Picture.Bitmap);
end;
*)

function DiskTypeStr(Drive: Char): ShortString;
var
  i: word;
begin
  i := GetDriveType(PChar(LowerCase(Drive+':\')));
  case i of
    { Onbekend }
    0: result := 'onbekend type';
    { bestaat niet }
    1: result := 'disk bestaat niet';
    { Floppy }
    DRIVE_REMOVABLE: result := 'floppy';
    { Harde Schijf }
    DRIVE_FIXED: result := 'harddisk';
    { Net eenheid }
     DRIVE_REMOTE: result := 'netwerk disk';
    { CD-ROM }
    DRIVE_CDROM: result := 'cd-rom';
    { Ram Drive }
    DRIVE_RAMDISK: result := 'ram-disk';
  end;

end;

function DiskInDrive(Drive: Char): Boolean;
var
  ErrorMode: word;
begin
  { maak er een hoofdletter van }
  Drive := UpCase(Drive);
  if not (Drive in ['A'..'Z']) then
    raise EConvertError.Create('Geen geldige stationsaanduiding: '+Drive+':');
  { schakel kritieke fouten uit }
  ErrorMode := SetErrorMode(SEM_FailCriticalErrors);
  try
    { drive 1 = a, 2 = b, 3 = c, etc. }
    if DiskSize(Ord(Drive) - Ord('A') + 1) = -1 then
      Result := False
    else
      Result := True;
  finally
    { restore old error mode }
    SetErrorMode(ErrorMode);
  end;
end;


function FreeLibrary(hLibModule : THANDLE) : longbool;
  stdcall;external 'kernel32' name 'FreeLibrary';
function LoadLibrary(lpLibFileName : pchar):THandle;
  stdcall;external 'kernel32' name 'LoadLibraryA';
function GetProcAddress(hModule : THandle;lpProcName : pchar) : pointer;
  stdcall;external 'kernel32' name 'GetProcAddress';


Type
  PFNSHGetFolderPath = Function(Ahwnd: HWND; Csidl: Integer; Token: THandle; Flags: DWord; Path: PChar): HRESULT; stdcall;


var
  SHGetFolderPath : PFNSHGetFolderPath = Nil;
  CFGDLLHandle : THandle = 0;

Procedure InitDLL;

Var
  P : Pointer;

begin
  CFGDLLHandle:=LoadLibrary('shell32.dll');
  if (CFGDLLHandle<>0) then
    begin
    P:=GetProcAddress(CFGDLLHandle,'SHGetFolderPathA');
    If (P=Nil) then
      begin
      FreeLibrary(CFGDLLHandle);
      CFGDllHandle:=0;
      end
    else
      SHGetFolderPath:=PFNSHGetFolderPath(P);
    end;
  If (P=Nil) then
    begin
    CFGDLLHandle:=LoadLibrary('shfolder.dll');
    if (CFGDLLHandle<>0) then
      begin
      P:=GetProcAddress(CFGDLLHandle,'SHGetFolderPathA');
      If (P=Nil) then
        begin
        FreeLibrary(CFGDLLHandle);
        CFGDllHandle:=0;
        end
      else
        ShGetFolderPath:=PFNSHGetFolderPath(P);
      end;
    end;
  If (@ShGetFolderPath=Nil) then
    Raise Exception.Create('Could not determine SHGetFolderPath Function');
end;

Function GetSpecialDir(ID :  Integer; const ForceCreate: Boolean = False) : String;
Var
  APath : Array[0..MAX_PATH] of char;
begin
  Result:='';
  if (CFGDLLHandle=0) then
  begin
    Try
      InitDLL;
    Except
      // SHGetFolderPath Function not available (Windows with IE version < 4)
      Exit;
    end;
  end;
  If (SHGetFolderPath<>Nil) then
    begin
      if ForceCreate then ID := ID or CSIDL_FLAG_CREATE;
      if SHGetFolderPath(0,ID,0,0,@APATH[0])=S_OK then
        Result:=IncludeTrailingPathDelimiter(StrPas(@APath[0]));
    end;
end;



Initialization

Finalization
 if CFGDLLHandle<>0 then
   FreeLibrary(CFGDllHandle);




end.

